"""Package info"""

__all__ = [
    '__package__', '__command__', '__version__', '__min_python_version__',
    '__min_git_version__', '__url__',
]

#: Package name
__package__ = 'git-workflow'
#: Base command
__command__ = 'workflow'
#: Package version
__version__ = '0.2.1'
#: Minimum required python version
__min_python_version__ = '3.6'
#: Minimum required git version for advanced features
__min_git_version__ = 2.23
#: Project URL
__url__ = 'https://github.com/connordelacruz/git-workflow'
